% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geneDotPlot.R
\name{geneDotPlot}
\alias{geneDotPlot}
\title{Plotting genes as dotplot}
\usage{
geneDotPlot(
  scdata,
  idents,
  genes,
  split.by = NULL,
  pct.threshold = 0.05,
  scale = NULL,
  standard_scale = NULL,
  keepLevels = TRUE,
  save.plot = FALSE,
  h = 5,
  w = 5,
  filepath = NULL,
  filename = NULL,
  heat_cols = NULL,
  col_limits = NULL,
  fill = FALSE,
  outline_col = "black",
  outline_size = 0.2
)
}
\arguments{
\item{scdata}{single-cell data. can be seurat/summarizedexperiment object}

\item{idents}{column name holding the idents for each cell}

\item{genes}{genes you want to plot}

\item{split.by}{column name in the metadata/coldata table to split the spots by. If not provided, it will plot via idents provided.}

\item{pct.threshold}{float. required to keep gene expressed by minimum percentage of cells}

\item{scale}{logical. scale the expression to mean +/- SD. NULL defaults to TRUE.}

\item{standard_scale}{logical. scale the expression to range from 0 to 1. NULL defaults to FALSE.}

\item{keepLevels}{logical. keep the original factor of the levels of the idents (for plotting)}

\item{save.plot}{logical. will try to save the pdf}

\item{h}{height of plot}

\item{w}{width of plot}

\item{filepath}{path to file, or path to folder}

\item{filename}{path to file}

\item{heat_cols}{colour gradient for the dot plot}

\item{col_limits}{set limits to the color gradient}

\item{outline_col}{colour of outlines if fill = TRUE}

\item{outline_size}{stroke size of outlines if fill = TRUE}
}
\value{
ggplot dot plot object of selected genes
}
\description{
Plotting genes as dotplot
}
\examples{
\donttest{
data(kidneyimmune)
geneDotPlot(kidneyimmune, genes = c("CD68", "CD80", "CD86", "CD74", "CD2", "CD5"), idents = "celltype", split.by = 'Project', standard_scale = TRUE) + theme(strip.text.x = element_text(angle=45, hjust = 0))
}
}
